from typing import Optional

from nexium_api import BaseRequestData, BaseResponseData, NexiumAPI, route, BaseRouter, BaseRequestAuth, ApiError


class RequestAuth(BaseRequestAuth):
    token: Optional[str]

    async def check(self):
        if self.token != 'string':
            raise ApiError(message='Only "string"!')


class Router(BaseRouter):
    request_auth = RequestAuth


class RequestData(BaseRequestData):
    pass


class ResponseData(BaseResponseData):
    pass


class WithParamsRequestData(BaseRequestData):
    first: int
    second: int


class WithParamsResponseData(BaseResponseData):
    sum: int


async def func():
    return ResponseData()


async def func_sum(first: int, second: int):
    return WithParamsResponseData(sum=first + second)


class ThirdRouter(Router):
    prefix = '/3'

    @route(
        path='/with-params',
        request_data=WithParamsRequestData,
        response_data=WithParamsResponseData,
        func=func_sum,
    )
    async def with_params(self):
        pass


class SecondRouter(Router):
    prefix = '/2'
    third_router: ThirdRouter


class HelloWorldRouter(Router):
    prefix = '/hello-world'

    second_router: SecondRouter

    @route(
        path='/get',
        request_data=RequestData,
        response_data=ResponseData,
        func=func,
    )
    async def get(self):
        pass


class MainRouter(Router):
    prefix = '/main'
    hello_world: HelloWorldRouter


async def main():
    await MainRouter().hello_world.second_router.third_router.with_params(first=1, second=2)


# run(main())

api = NexiumAPI(
    main_router=MainRouter,
    contact={
        'name': 'Nexium',
        'url': 'https://nexium.me',
        'email': 'info@nexium.me',
    },
    license_info={
        'name': 'Apache 2.0',
        'url': 'https://www.apache.org/licenses/LICENSE-2.0.html',
    },
)
